#!/usr/bin/env python
# -*- coding: iso-8859-2 -*-
'''
Copyright (C) 2008 Jarosaw Krzywicki, krzywy77@gmail.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''

import sys, math, getopt, os.path #, inkex
from bezmisc import bezierpointatt, bezierlengthSimpson
from pathmodifier import composeTransform, applyTransformToPoint

tr = [ 0.0, 0.0 ]
sc = [ 0.0, 0.0 ]

point_t = 25.4 / 90

b_path = [ -1.0, -1.0 ]
e_path = [ -1.0, -1.0 ]
z_val = 0.0
z_up = True
workz = 0.0
speedz = 0.0
bezlen = 2.0
min_bez_seg = 4
speed_pcm = 0.0
speed_tol = 0.0
work_pcm = 0.0
work_tol = 0.0

def translate_and_scale( pt ) :
	#pt[ 0 ] = pt[ 0 ] + tr[ 0 ]
	#pt[ 1 ] = pt[ 1 ] + tr[ 1 ]
	#pt[ 0 ] = round( pt[ 0 ] * sc[ 0 ] * point_t, 3 )
	#pt[ 1 ] = round( pt[ 1 ] * sc[ 1 ] * point_t, 3 )
	pt[ 0 ] = round( pt[ 0 ] * point_t, 3 )
	pt[ 1 ] = round( pt[ 1 ] * - point_t, 3 )

class StackCTM :
	def __init__( self ) :
		self.stack = []
		self.stack.append( [ [ 1.0, 0.0, 0.0 ], [ 0.0, 1.0, 0.0 ] ] )

	def gsave( self ) :
		m1 = self.stack[ -1 ]
		self.stack.append( m1 )

	def grestore( self ) :
	    #if len( self.stack ) :
	    self.stack.pop()
	    #else :
	    #    sys.stderr.write( "STACK IS EMPTY" )
	
	def concat( self, m2 ) :
		m1 = self.stack.pop()
		self.stack.append( composeTransform( m1, m2 ) )
		##self.stack.append( m2 )

	def push( self, m2 ) :
		self.stack.append( m2 )

	def pop( self ) :
		return self.stack.pop()

	def top( self ) :
		return self.stack[ -1 ]


stack_ctm = StackCTM()


def gc_Begin( outfile ) :
        outfile.write( 'G17 G21 G40 G49 G54 G80 G90 G94\nF100\n' )

def gc_End( outfile ) :
        outfile.write( 'M2 (EOF)\n' )

def gc_Z_up( outfile ) :
    global z_up, speedz, speed_pcm, speed_tol
    if not z_up :
	z_up = True
	outstr = ''
	if speed_pcm == 61.0 :
	    outstr = '( UP )\nG1 Z0.000\nG61\nG0 Z' + str( speedz ) + '\n'
	elif speed_pcm == 61.1 :
	    outstr = '( UP )\nG1 Z0.000\nG61.1\nG0 Z' + str( speedz ) + '\n'
	elif speed_pcm == 64.0 :
	    outstr = '( UP )\nG1 Z0.000\nG64 P' \
		     + str( speed_tol ) + \
		     '\nG0 Z' + str( speedz ) + '\n'
	outfile.write( outstr )

def gc_Z_down( outfile ) :
    global z_up, workz, work_pcm, work_tol
    if z_up :
	z_up = False
	outstr = ''
	if work_pcm == 61.0 :
	    outstr = '( UP )\nG0 Z0.000\nG61\nG1 Z' + str( workz ) + '\n'
	elif work_pcm == 61.1 :
	    outstr = '( UP )\nG0 Z0.000\nG61.1\nG1 Z' + str( workz ) + '\n'
	elif work_pcm == 64.0 :
	    outstr = '( UP )\nG0 Z0.000\nG64 P' \
		     + str( work_tol ) + \
		     '\nG1 Z' + str( speedz ) + '\n'
	outfile.write( outstr )


def gc_G0( outfile, pt ) :
    for i in range( 0, len( pt ) / 2, 2 ) :
	p = [ pt[ i ], pt[ i + 1 ] ]
	applyTransformToPoint( stack_ctm.top(), p )
	translate_and_scale( p )
        outstr = 'G0 X' + str( p[ 0 ] ) + ' Y' + str( p[ 1 ] ) + '\n'
        outfile.write( outstr )

def gc_G1( outfile, pt ) :
    for i in range( 0, len( pt ) / 2, 2 ) :
	p = [ pt[ i ], pt[ i + 1 ] ]
	applyTransformToPoint( stack_ctm.top(), p )
	translate_and_scale( p )
        outstr = 'G1 X' + str( p[ 0 ] ) + ' Y' + str( p[ 1 ] ) + '\n'
        outfile.write( outstr )

def gc_G2G3( outfile, pt ) :
    #for i in range( 0, len( pt ) / 2, 2 ) :
	#p = [ pt[ i ], pt[ i + 1 ] ]
	#applyTransformToPoint( stack_ctm.top(), p )
	#translate_and_scale( p )
	#outstr = 'G1 X' + str( p[ 0 ] ) + ' Y' + str( p[ 1 ] ) + '\n'
	#outfile.write( outstr )
	# pierwszy punkt powinien znajdowa si w e_path
    pt0 = [ e_path[ 0 ], e_path[ 1 ] ]
    pt1 = [ pt[ 0 ], pt[ 1 ] ]
    pt2 = [ pt[ 2 ], pt[ 3 ] ]
    pt3 = [ pt[ 4 ], pt[ 5 ] ]
    applyTransformToPoint( stack_ctm.top(), pt0 )
    applyTransformToPoint( stack_ctm.top(), pt1 )
    applyTransformToPoint( stack_ctm.top(), pt2 )
    applyTransformToPoint( stack_ctm.top(), pt3 )
    translate_and_scale( pt0 )
    translate_and_scale( pt1 )
    translate_and_scale( pt2 )
    translate_and_scale( pt3 )
    l = bezierlengthSimpson( ( pt0, pt1, pt2, pt3 ) )
    t = int( l / bezlen )
    if t < min_bez_seg :
	t = min_bez_seg
    n = 1.0 / t
    outstr = '( BEZIER SEGMENT - ' + str( t ) + ' ELEMENTS )\n'
    outfile.write( outstr )
    for i in range( t ) :
	t = n + i * n
	p = bezierpointatt( ( pt0, pt1, pt2, pt3 ), t )
	outstr = 'G1 X' + str( p[ 0 ] ) + ' Y' + str( p[ 1 ] ) + '\n'
	outfile.write( outstr )
    

def split_by_space( string ) :
    strl = string.lstrip()  # obetnij puste znaki na pocztku
    strlr = string.rstrip() # i na kocu
    lst = strlr.split( ' ' )
    for i in range( len( lst ) ) :
        lst[ i ] = float( lst[ i ] )
    return lst

# Docelowo: Wrzuci macierz na stos
def concat( debugfile, string ) :
    b = line.find( '[' )
    e = line.rfind( ']' )
    l = split_by_space( string[ b + 1 : e ] )
    gsave( debugfile, string )
    stack_ctm.concat( [ [ l[0], l[2], l[4] ], [ l[1], l[3], l[5] ] ] )

    debugfile.write( str( stack_ctm.top() ) )
    debugfile.write( ' - CONCAT\n' )

def gsave( debugfile, string ) :
    stack_ctm.gsave()
    debugfile.write( 'SAVE - ' )

def grestore( debugfile, string ) :
    stack_ctm.grestore()
    debugfile.write( ' - RESTORE\n' )

def translate( debugfile, string ) :
    e = line.find( 'translate' )
    tr[:] = split_by_space( string[ : e ] )
    #stack_ctm.concat( [ [ 1.0, 1.0, l[0] ], [ 1.0, 1.0, l[1] ] ] )
    debugfile.write( str( tr ) )
    debugfile.write( ' - translate\n' )

def scale( debugfile, string ) :
    e = line.find( 'scale' )
    sc[:] = split_by_space( string[ : e ] )
    #stack_ctm.concat( [ [ l[0], 1.0, 1.0 ], [ 1.0, l[1], 1.0 ] ] )
    debugfile.write( str( sc ) )
    debugfile.write( ' - scale\n' )

def newpath( debugfile, outfile, string ) :
    b_path[:] = [ -1.0, -1.0 ]
    debugfile.write( 'NEW\n' )

    gc_Z_up( outfile )

def closepath( debugfile, outfile, string ) :
    gc_G1( outfile, b_path )

    gc_Z_up( outfile )

    b_path[:] = [ -1.0, -1.0 ]
    e_path[:] = [ -1.0, -1.0 ]
    debugfile.write( 'CLOSE\n' )

def moveto( debugfile, outfile, string ) :
    e = line.find( 'moveto' )
    b_path[:] = split_by_space( string[ : e ] )
    debugfile.write( str( b_path ) )
    debugfile.write( ' - MOVE\n' )

    gc_Z_up( outfile )

    gc_G0( outfile, b_path )
    e_path[:] = b_path
    
def lineto( debugfile, outfile, string ) :
    e = line.find( 'lineto' )
    my_line = split_by_space( string[ : e ] )
    debugfile.write( str( my_line ) )
    debugfile.write( ' - LINE\n' )

    gc_Z_down( outfile )

    gc_G1( outfile, my_line )
    e_path[:] = [ my_line[ -2 ],  my_line[ -1 ] ]

def curveto( debugfile, outfile, string ) :
    e = line.find( 'curveto' )
    my_curve = split_by_space( string[ : e ] )
    debugfile.write( str( my_curve ) )
    debugfile.write( ' - CURVE\n' )

    gc_Z_down( outfile )

    gc_G2G3( outfile, my_curve )
    e_path[:] = [ my_curve[ -2 ],  my_curve[ -1 ] ]


args = len( sys.argv )

if args < 3 :
    sys.exit( -3 )
debug_path = '~/.inkscape/debug_ps2ngc.txt'
debug_path = os.path.expanduser( debug_path ) ;
debug = open( debug_path, 'w' )
## argv[0] - nazwa programu, argv[1 ... len - 1] - argumenty
for arg in sys.argv :
    debug.write( arg + ', ' )
debug.write( '\n\n' )

optlist, args = getopt.getopt( sys.argv[1:], '', \
			       ['speedz=', 'workz=', 'bezlen=', \
				'speed_pcm=' ,'speed_tol=', \
				'work_pcm=' , 'work_tol=', \
				'min_bez_seg=' ] )
for a, v in optlist :
    if a == "--speedz" :	# Warto Z dla ruchw szybkich
        speedz = float( v )
    elif a == "--workz" :	# Warto Z dla ruchw roboczych
        workz = float( v )
    elif a == "--bezlen" :	# rednia dugo odcinkw na jakie zostanie
        bezlen = float( v )	# podzielony segment Bezier-a
    elif a == "--min_bez_seg" :	# Min. ilo odcinkw z jednego segm. Bezier-a
        min_bez_seg = int( v )
    elif a == "--speed_pcm" :	# 'Path Control Mode' ruchw szybkich
        if v == 'G61 (exact path mode)' :
	    speed_pcm = 61.0
        elif v == 'G61.1 (exact stop mode)' :
	    speed_pcm = 61.1
        elif v == 'G64 Tolerance (continuous mode with tolerance)' :
	    speed_pcm = 64.0
    elif a == "--speed_tol" :	# Tolerancja dla szybkich 'continuous mode'
        speed_tol = float( v )
    elif a == "--work_pcm" :	# 'Path Control Mode' ruchw roboczych
        if v == 'G61 (exact path mode)' :
	    work_pcm = 61.0
        elif v == 'G61.1 (exact stop mode)' :
	    work_pcm = 61.1
        elif v == 'G64 Tolerance (continuous mode with tolerance)' :
	    work_pcm = 64.0
    elif a == "--work_tol" :	# Tolerancja dla roboczych 'continuous mode'
        work_tol = float( v )
    debug.write( a + ', ' + v + ': ' )
debug.write( '\n' )

for arg in args :
    debug.write( arg + ', ' )
debug.write( '\n\n' )

input_f = open( args[ 0 ], 'r' )
output_f = sys.stdout

gc_Begin( output_f )

for line in input_f :
    if line.find( 'concat' ) >= 0 :
        concat( debug, line )
    elif line.find( 'gsave' ) >= 0 :
        gsave( debug, line )
    elif line.find( 'grestore' ) >= 0 :
        grestore( debug, line )
    elif  line.find( 'translate' ) >= 0 :
        translate( debug, line )
    elif  line.find( 'scale' ) >= 0 :
        scale( debug, line )
    elif  line.find( 'newpath' ) >= 0 :
        newpath( debug, output_f, line )
    elif  line.find( 'closepath' ) >= 0 :
        closepath( debug, output_f, line )
    elif  line.find( 'moveto' ) >= 0 :
        moveto( debug, output_f, line )
    elif  line.find( 'lineto' ) >= 0 :
        lineto( debug, output_f, line )
    elif  line.find( 'curveto' ) >= 0 :
        curveto( debug, output_f, line )
    else :
        val =  '???     ' + line
        #debug.write( val )


gc_End( output_f )

debug.close()
input_f.close()
